***************************************************
FUNCTION GetCheck
LPARAMETER lcNum, lcChkDigit
*-------------------------------------------------------
* Function.....: GetCheck
* Written by...: Michael P. Antonovich
*
* Abstract.....: Calculates the check digit for any string
*                If a check digit is passed, routine validates the check digit,
*                     lcChkDigit, against the number, lcNum
*                If no check digit is passed, routine calculates and returns
*                     the check digit
*
* Returns......: If no validation, One character check digit as a string character
*                If validation, .T. if check digit passed matches calculation, else .F.
*
* Parameters...: lcNum - Number or string passed to this routine
*                lcChkDigit - Character or number check digit to validate
*
* Notes........: This function performs both the calculation of the
*                check digit for any number or character string as well
*                as the validation of a check digit against a number or string
*-------------------------------------------------------
LOCAL lcChkDigit, GetCheck, lcOldStr, lcNewStr, lcNum, lcWeight
LOCAL lni, lnChrCnt, lnMode, lnTotal

*  Define Weight String
  lcWEIGHT = "234567234567234567234567234567234567"
  lcOldStr = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  lcNewStr = "012345678912345678912345678923456789"
  lnMode = PARAMETERS()

  DO CASE
    CASE TYPE("num") = "N"
      lcnum = ALLTRIM(STR(num,20))
    CASE TYPE("num") = "C"
      lcnum = CHRTRAN(num, lcOldStr, lcNewStr)
    OTHERWISE
      RETURN " "
  ENDCASE

* Build array to calculate factors
  lnChrCnt = LEN(lcNum)
  lnTotal  = 0
  DIMENSION AMPA[lnChrCnt,3]
  FOR lnI = 1 to lnChrCnt
     AMPA[lnI,1] = VAL(SUBSTR(lcNum,lnI,1))
     AMPA[lnI,2] = VAL(SUBSTR(lcWeight,lnChrCnt-lnI+1,1))
     AMPA[lnI,3] = AMPA[lnI,1] * AMPA[lnI,2]
     lnTotal = lnTotal + AMPA[lnI,3]
  ENDFOR &&* lnI = 1 to lnChrCnt

  IF lnMode = 1
    GetCheck = STR(MOD(lnTotal,10), 1)
  ELSE
    DO CASE
      CASE TYPE("lcChkDigit") = 'N'
        lcChkDigit = STR(lcChkDigit,1)
    ENDCASE
   GetCheck = (STR(MOD(lnTotal,10), 1) = lcChkDigit)
  ENDIF

RETURN GetCheck
ENDFUNC &&* GetCheck


***************************************************
PROCEDURE OPENDBC
*--------------------------------------------------
* Function....: OpenDBC
* Called by...: Varies
*
* Abstract....: When opening a DBC, checks first if DBC is already open,
*               if so, just sets it as current
*
* Returns.....: N/A
*
* Parameters..: Full File name (including drive/path) of DBC
*
* Notes.......:  
*-------------------------------------------------------
LPARAMETER lcFullName
LOCAL lcSetDBC, lnElement, lcCurExact
LOCAL ARRAY laDBCList(1)

  lcCurExact = SET("EXACT")
  SET EXACT ON

* Get list of open DBCs
  = ADATABASES(laDBCList)

* Search for Full DBC name in list
  lnElement = ASCAN(laDBCList, lcFullName)

  IF lnElement = 0
  * Not found among open databases
    OPEN DATABASE (lcFullName)
  ELSE
  * Set database to selected
    lcSetDBC = laDBCList(lnElement-1)
    SET DATABASE TO (lcSetDbc)
  ENDIF

  SET EXACT &lcCurExact
RETURN


***************************************************
PROCEDURE OPENFILE
*--------------------------------------------------
* Function....: OpenFile
* Called by...: Varies
*
* Abstract....: When opening a DBF, checks first if file is already open,
*               if so, just sets current area to it
*
* Returns.....: 
*
* Parameters..: Full File name (including drive/path)
*
* Notes.......:  
*-------------------------------------------------------
LPARAMETER lcFullName
LOCAL lcAlias, lcCurError

  lcFullName = UPPER(ALLTRIM(lcFullName))
  IF EMPTY(lcFullname)
    oApp.ErrStr = "No filename has been specified!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Bad FileName")
    ENDIF
    oApp.Success = .F.
    RETURN .F.
  ELSE
    lcFullName = lcFullName + IIF(RIGHT(lcFullName,4) = ".DBF", "", ".DBF")
  ENDIF

* Get alias   && Assume you are using VFP 6.0 or have included the FOXTOOLS library
  lcAlias = JUSTSTEM(lcFullName)

* Open file if not currently open, then select work area
  IF USED(lcAlias)
    SELECT (lcAlias)
  ELSE
    lcCurError = ON("ERROR")
    ON ERROR DO NOFILE WITH ERROR(), lcAlias
    SELECT 0
    IF FILE(lcFullName)
      USE (lcFullName) AGAIN ALIAS &lcAlias
    ELSE
      USE (lcAlias) AGAIN ALIAS &lcAlias
    ENDIF
    ON ERROR &lcCurError
  ENDIF
RETURN oApp.Success


***************************************************
PROCEDURE NOFILE
*--------------------------------------------------
LPARAMETER lnErrNo, lcAlias
  DO CASE
    CASE lnErrNo = 1
      oApp.ErrStr = "File: " + ALLTRIM(lcAlias) + " cannot be found."
      IF oApp.Silent
*        WAIT WINDOW oApp.ErrStr TIMEOUT 15
      ELSE
        MESSAGEBOX(oApp.ErrStr, 16, "Missing File")
      ENDIF
      oApp.Success = .F.
      RETURN TO MASTER
  ENDCASE
RETURN


***************************************************
FUNCTION CLOSFILE
*--------------------------------------------------
* Function....: ClosFile
* Called by...: Varies
*
* Abstract....: Closes the file passed to it if open
*
* Returns.....: True of the file was open, False if it was not
*
* Parameters..: File alias or full filename
*
* Notes.......:  
*-------------------------------------------------------
LPARAMETER lcAliasName

* If no alias/filename supplied, return .F.
  IF EMPTY( lcAliasName )
    RETURN .F.
  ELSE

  * check if the passed name is a full filename
    IF AT(".", lcAliasName) > 0 OR AT("\", lcAliasName) > 0 OR AT("/", lcAliasName) > 0
      lcAliasName = JUSTSTEM(lcAliasName)
    ENDIF

  * IF open close the file
    IF USED(lcAliasName)
      USE IN (lcAliasName)
    ELSE
      RETURN .F.
    ENDIF
  ENDIF
RETURN .T.


***************************************************
FUNCTION DELFILE
*--------------------------------------------------
* Function....: DelFile
* Called by...: Varies
*
* Abstract....: Closes and deletes the file passed to it
*
* Returns.....: True if the file was deleted, False if it was not
*
* Parameters..: File alias or full filename
*
* Notes.......:  
*-------------------------------------------------------
LPARAMETER lcAliasName
LOCAL lcFileLoc

* If no alias/filename supplied, return .F.
  IF EMPTY( lcAliasName )
    RETURN .F.
  ELSE

  * check if the passed name is a full filename
    IF AT(".", lcAliasName) > 0 OR AT("\", lcAliasName) > 0 OR AT("/", lcAliasName) > 0
      lcAliasName = JUSTSTEM(lcAliasName)
    ENDIF

  * Based on alias, check if file is used.  If so, select it to get full filename
    IF USED( lcAliasname )
      SELECT ( lcAliasName )
    ELSE
      RETURN .F.
    ENDIF
  ENDIF

* Capture complete location of the file
  lcFileLoc = DBF()

* Close and erase file
  USE
  ERASE ( lcFileLoc )
RETURN .T.


*-------------------------------------------------------
* Function.....:  PackFile
* Written by...:  Michael P. Antonovich - Friday, November 13, 1998
*
* Abstract.....:  Packs a table
*
* Returns......:  None
*
* Parameters...:  cFile
*
* Notes........:  I'm throwing this together quickly to test an 
*                 automation stream.  Probably need some checks
*                 added in to verify/specify a table or alias
*                 in use.
*-------------------------------------------------------
PROCEDURE PackFile
LPARAMETERS lcFile
LOCAL lnDelCnt, lcAliasName

* Capture old on error
  curError  = ON("ERROR")
  curDelete = SET("DELETED")
  curArea   = ALIAS()
  SET DELETED OFF

* Open the file to be packed and set it as the active work area
  = OPENFILE(lcFile)
  COUNT FOR DELETED() TO lnDelCnt
  SET DELETED &curDelete

  IF lnDelCnt > 0
  * Get alias name of file to be packed.
    lcAliasName = JUSTSTEM(lcFile)

  * Get file full name and open exclusively to pack
    ERRCNT = 0
    ON ERROR DO PACKERROR WITH ERROR(), lcFile
    a = DBF()
    curTag = SYS(22)
    USE (A) EXCLUSIVE
    IF oApp.ErrorNum = 0

      PACK
      USE (A) SHARED
      IF !EMPTY(curTag)
        SET ORDER TO TAG &curTag
      ENDIF
  
    ELSE
      oApp.ErrorNum = -9     && Exclusive access failed.
    ENDIF

  * Reset on error
    ON ERROR &curError
    IF !EMPTY(curArea)
      SELECT (curArea)
    ENDIF
  ENDIF

RETURN


*-------------------------------------------------------
* Function.....:  PackError
* Written by...:  Michael P. Antonovich, Friday, November 13, 1998
*
* Abstract.....:  An error has occurred during a pack routine
*
* Returns......:  Could return to PROCEDURE DRIVER if it cannot get exclusive access
*
* Parameters...:  ERRNUM - Error Number
*                 lcFile - File Name of file in which pack error occurred
*
* Notes........:  
*                 
*-------------------------------------------------------
PROCEDURE PACKERROR
LPARAMETERS ERRNUM, lcFile

  ERRCNT = ERRCNT + 1
  DO CASE
    CASE ERRNUM = 3
    * Allow program to automatically try up to 500 times.
      IF ERRCNT >= 500
        oApp.ErrorNum = -3
        RETURN
      ELSE
      * Retry every 5 seconds
        WAIT WINDOW "Cannot get exclusive access to: " + lcFile NOWAIT
        now = SECONDS()
        DO WHILE SECONDS()-NOW < 5
        ENDDO
        RETRY
      ENDIF
  ENDCASE
     
RETURN


**************************************************
FUNCTION XSWAP
*--------------------------------------------------
* Function....: XSWAP
* Called by...: Varies
*
* Abstract....: Switches the contents of the two passed parameters
*
* Returns.....: T if successful
*
* Parameters..: 
*
* Notes.......: Must pass parameters by reference:   = xswap(@a,@b)
*
* Author......: Michael P. Antonovich (06/01/1996)
*-------------------------------------------------------
LPARAMETERS a, b
LOCAL xTemp, xSuccess

* Only works if there are two parameters
  IF PARAMETERS() = 2
  * Swap values
    xTemp = a
    a     = b
    b     = xTemp
    xSuccess = .T.
  ELSE
    xSuccess = .F.
  ENDIF
RETURN xSuccess


**************************************************
FUNCTION TAGEXIST
*--------------------------------------------------
* Function....: TAGEXIST
* Called by...: Varies
*
* Abstract....: Determines whether a specific tag exists in the structured index of the current or specified table
*
* Returns.....: .T. if it finds the tag
*               .F. if it does not find the tag
*
* Parameters..: First: Name of the tag to search for
*               Second: Name of the DBF to search in the corresponding CDX
*               Third: Name of the DBC which contains the DBF
*
* Notes.......: Sample call: 
*
* Author......: Michael P. Antonovich (07/20/1998)
*-------------------------------------------------------
LPARAMETERS lcTagName, lcDBFName, lcDBCName
LOCAL lcCurArea, lcDBC, llExist, lnCount

* Initialize
  llExist = .F.

* Open correct table
  lcCurArea = ALIAS()
  IF PARAMETERS() = 3 AND !EMPTY(lcDBCName)
    lcDBC = ALLTRIM(lcDBCName) + "!"
  ELSE
    lcDBC = ""
  ENDIF
  IF !EMPTY(lcDBFName)
    IF USED(lcDBFName)
      SELECT (lcDBFName)
    ELSE
      SELECT 0
      USE (lcDBC+lcDBFName)
    ENDIF
  ENDIF

* Read through the tags in the corresponding structured index of current table
  FOR lnCount = 1 TO 254
    IF !EMPTY(TAG(lnCount))
      IF TAG(lnCount) = UPPER(lcTagName)
        llExist = .T.
        EXIT    && Tag found, no need to search further
      ENDIF
    ELSE
      EXIT    && No more tags, searched for tag not found
    ENDIF
  ENDFOR

* Reset environment
  IF !EMPTY(lcCurArea)
    SELECT (lcCurArea)
  ENDIF

RETURN llExist


**************************************************
PROCEDURE SETDEFAULTDIR
**************************************************
* Program....: SETDEFAULTDIR
* Version....: 1.0
* Author.....: Michael P. Antonovich
* Date.......: June 21, 1999
* Notice.....: Copyright (c) 1999, MicMin Associates. All Rights Reserved.
* Compiler...: Visual FoxPro 05.00.00.0402 for Windows 
* Abstract...: Extracts the path from the current executable in VFP 5.0
* Changes....:
* Notes......: This routine will not work if directory names have spaces in them.
**************************************************
LOCAL lcDir, lcDrive, lcPath, lnSpace, lnSlash

  lcDir = SYS(16,0)
  IF RIGHT(lcDir,3) $ "FXP-SCT"

  * Read last 'word' in string returned by SYS(16,0)
    lnSpace = RAT(" ", lcDir)
    IF lnSpace > 0
      lcDir = SUBSTR(lcDir, lnSpace+1)
    ENDIF

  * Attempt to use FOXTOOLS if available
    IF FILE(SYS(2004) + "FOXTOOLS.FLL")
      SET LIBRARY TO SYS(2004) + "FOXTOOLS.FLL" ADDITIVE
      lcPath  = ADDBS(JUSTPATH(lcDir))
      IF SUBSTR(lcPath,2,1) = ":"
        lcDrive = left(lcPath, 2)
        lcPath = SUBSTR(lcPath, 3)
      ELSE
        lcDrive = JUSTDRIVE(lcDir)
      ENDIF
    ELSE

    * Check for a drive letter
      IF SUBSTR(lcDir, 2, 1) = ":"
        lcDrive = LEFT(lcDir, 2)
        lcPath  = SUBSTR(lcDir, 3)
      ELSE
        lcDrive = SYS(5)
        lcPath  = lcDir
      ENDIF

    * Remove program name to get path
      lnSlash = RAT("\", lcPath)
      IF lnSlash > 0
        lcPath  = SUBSTR(lcPath, 1, lnSlash)
      ELSE
        lcPath = ""
      ENDIF

    ENDIF
  ENDIF
  SET DEFAULT TO &lcdrive.&lcpath


**************************************************
FUNCTION GetStructure
*--------------------------------------------------
* Function....: GetStructure
* Called by...: Varies
*
* Abstract....: Prints the structures of all tables referenced by the DBC in the
*               default application data directory.
*
* Returns.....: 
*
* Parameters..: none
*
* Notes.......: = GetStructure
*
* Author......: Michael P. Antonovich (05/30/1999)
*-------------------------------------------------------
LOCAL lcHomeDir, lnDBCCount, lcOrigDir, lcDBCName, lnI, lnJ, llIsDBCOpen, llIsDBFOpen, lnTableCount

  lcOrigDir = SYS(5) + CURDIR()
  
* Get home directory for current application and append data path
  lcHomeDir   = JUSTPATH(SYS(16,1)) + "\DBFS"

* Switch to data directory and get a list of all DBC's in that directory
  CD &lcHomeDir
  lnDBCCount = ADIR(aDBCList, "*.DBC")

* Step through the DBC's to find all the tables in each.
  FOR lnI = 1 TO lnDBCCount
    lcDBCName = aDBCList[lnI,1]

  * Select the DBC if already open, open it if not
    llIsDBCOpen = OPENDBC(lcDBCName)    && Returns .T. if DBC is already open
  
  * Step through the tables in the dbc
    lnTableCount = ADBOBJECTS(atables, "TABLE")
    FOR lnJ = 1 TO lnTableCount
      llIsDBFOpen = OPENFILE(aTables[lnJ])
      DISPLAY STRUCTURE
    ENDFOR

  * Close database and all of its tables if not previously open
    IF !llIsDBCOpen
      CLOSE DATABASES
    ENDIF

  ENDFOR
  CD &lcOrigDir
RETURN

**************************************************
FUNCTION INPUTBOX
*--------------------------------------------------
* Function....: INPUTBOX
* Called by...: Varies
*
* Abstract....: Opens a window and prompts user for a simple response
*
* Returns.....: User response to prompt
*
* Parameters..: 
*
* Notes.......: Michael P. Antonovich (05/13/1998)
*-------------------------------------------------------
LPARAMETERS lcDesc, lcTitle, lcDefault, lnXPos, lnYPos
LOCAL lcStr

* Initialize
  lcStr = ""

* Open ResponseBox class
  IF "DEMOCLASS" $ SET("CLASSLIB")
    o_Prompt = CREATEOBJECT("ResponseBox", lcDesc, lcTitle, lcDefault, lnXPos, lnYPos)
    o_Prompt.show(1)
    lcStr = o_prompt.prompt
    release o_prompt
  ENDIF

RETURN lcStr


PROCEDURE DAILYLOAD
*************************************************************************
*
* Program:       DAILYLOAD
* Written By:    Michael P. Antonovich
*                Copyright 1999, MicMin Associates
* Written On:    July 5, 1999
* Parameters:
*   lcRawData  - Fully qualified name of raw text data file
*   lcTempFile - Fully qualified name of temporary data file
*   lcType     - Type of raw data file:
*                    1 - DBF file
*                    2 - Delimited file
*                    3 - SDF file
*                    4 - Excel spreadsheet, type XLS
* Returns:       Nothing
*
************************************************************************
LPARAMETERS lcRawData, lcTempFile, lcType

* Check if three parameters were passed.
* If only two were passed, default the type to '1'
* If only one was passed, set error and return
  DO CASE
    CASE PARAMETERS() = 2
      lcType = '2'
    CASE PARAMETERS() < 2
      oApp.ErrStr = "One or more required parameters have not been passed."
      IF oApp.Silent
*        WAIT WINDOW oApp.ErrStr TIMEOUT 15
      ELSE
        MESSAGEBOX(oApp.ErrStr, 16, "Error in number of parameters")
      ENDIF
      oApp.Success = .F.
      RETURN
  ENDCASE

* Check if the raw data file exists.  Otherwise, this is pointless.
  IF !FILE(lcRawData)
    oApp.ErrStr = "Raw Data File is not found!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Data File")
    ENDIF
    oApp.Success = .F.
    RETURN
  ENDIF
  
* Check if the output data file exists.  Production system would create the
* temporary file with a CREATE TABLE command.
  IF !FILE(lcTempFile)
    oApp.ErrStr = "Output Data File is not found!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Output File")
    ENDIF
    oApp.Success = .F.
    RETURN
  ELSE
    = OPENFILE(lcTempFile)
  ENDIF

* Append data based on the appropriate file type
  DO CASE
    CASE lcType = '1'        && DBF
      APPEND FROM (lcRawData)
    CASE lcType = '2'        && Delimited
      APPEND FROM (lcRawData) DELIMITED
    CASE lcType = '3'        && Type SDF
      APPEND FROM (lcRawData) TYPE SDF
    CASE lcType = '4'        && Excel File
      APPEND FROM (lcRawData) TYPE XLS
  ENDCASE

* Close data file
  = CLOSFILE(lcTempFile)

RETURN


PROCEDURE INTRADEDUPE
*************************************************************************
*
* Program:       INTRADEDUPE
* Written By:    Michael P. Antonovich
*                Copyright 1999, MicMin Associates
* Written On:    July 5, 1999
* Parameters:
*   lcTempFile - Fully qualified name of temporary data file
* Returns:       Nothing
*
************************************************************************
LPARAMETERS lcTempFile
LOCAL lcNewFile

* If lcTempFile is not passed, exit the routine, else, get the filename stem
  IF EMPTY(lcTempFile)
    oApp.ErrStr = "Required temporary filename parameter is missing!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Parameter")
    ENDIF
    oApp.Success = .F.
    RETURN
  ELSE
    lcTempStem = JUSTSTEM(lcTempFile)
  ENDIF

* If lcTempFile does not exist, exit the routine
  IF !FILE(lcTempFile)
    oApp.ErrStr = "Expected file " + AlLTRIM(lcTempFile) + " does not exist!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing File")
    ENDIF
    oApp.Success = .F.
    RETURN
  ENDIF

* Generate a new table name
  lcNewFile = ADDBS(JUSTPATH(lcTempFile)) + SYS(3) + ".DBF"

* Select the unique records only
  SELECT * ;
    FROM (lcTempFile) ;
   GROUP BY cZip, cAddress, cLName, cFName ;
    INTO TABLE (lcNewFile)
  USE

* Delete the old table
  IF USED(lcTempStem)
    USE IN (lcTempStem)
  ENDIF
  ERASE (lcTempFile)

* Rename new table to old table name
  RENAME (lcNewFile) TO (lcTempFile)

RETURN


PROCEDURE DELBADRECS
*************************************************************************
*
* Program:       DELBADRECS
* Written By:    Michael P. Antonovich
*                Copyright 1999, MicMin Associates
* Written On:    July 5, 1999
* Parameters:
*   lcTempFile - Fully qualified name of temporary data file
* Returns:       Nothing
*
************************************************************************
LPARAMETERS lcTempFile
LOCAL lcNewFile

* If lcTempFile is not passed, exit the routine, else, get the filename stem
  IF EMPTY(lcTempFile)
    oApp.ErrStr = "Required temporary filename parameter is missing!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Parameter")
    ENDIF
    oApp.Success = .F.
    RETURN
  ELSE
    lcTempStem = JUSTSTEM(lcTempFile)
  ENDIF

* If lcTempFile does not exist, exit the routine
  IF !FILE(lcTempFile)
    oApp.ErrStr = "Expected file " + ALLTRIM(lcTempFile) + " does not exist!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing File")
    ENDIF
    oApp.Success = .F.
    RETURN
  ENDIF

* Select the unique records only
  DELETE FROM (lcTempFile) ;
   WHERE EMPTY(cLName) OR ;
         EMPTY(cAddress) OR ;
         EMPTY(cCity) OR ;
         EMPTY(cState) OR ;
         EMPTY(cZip)

RETURN


PROCEDURE MERGERECS
*************************************************************************
*
* Program:       MERGERECS
* Written By:    Michael P. Antonovich
*                Copyright 1999, MicMin Associates
* Written On:    July 5, 1999
* Parameters:
*   lcOrigFile - Name of table you want to merge records into
*   lcTempFile - Fully qualified name of temporary data file
* Returns:       Nothing
*
************************************************************************
LPARAMETERS lcOrigFile, lcTempFile

* Check if both parameters have been passed.
  IF PARAMETERS() < 2
    oApp.ErrStr = "One or more required parameters have not been passed"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Error in number of parameters")
    ENDIF
    oApp.Success = .F.
    RETURN
  ENDIF

* Check if the raw data file exists.  Otherwise, this is pointless.
  IF !FILE(lcOrigFile)
    oApp.ErrStr = "Data File to merge into is not found!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Data File")
    ENDIF
    oApp.Success = .F.
    RETURN
  ELSE
    = OPENFIlE(lcOrigFile)
    USE (DBF()) ALIAS OrigFile
  
  * Get maximum customer code value in table
    IF RECCOUNT() > 0
      SELECT MAX(cCustCode) AS cMax ;
        FROM (lcOrigFile) ;
        INTO CURSOR MAXVAL
      lnMax = VAL(MAXVAL.cMax)
      USE IN MAXVAL
    ELSE
      lnMax = 0
    ENDIF
  ENDIF
  
* Check if the merge data file exists.
  IF !FILE(lcTempFile)
    oApp.ErrStr = "Merge Data File is not found!"
    IF oApp.Silent
*      WAIT WINDOW oApp.ErrStr TIMEOUT 15
    ELSE
      MESSAGEBOX(oApp.ErrStr, 16, "Missing Merge File")
    ENDIF
    oApp.Success = .F.
    RETURN
 
  ELSE
    = OPENFILE(lcTempFile)
    USE (DBF()) EXCLUSIVE

  * Add missing fields to table
    ALTER TABLE TEMPCUST ADD COLUMN cCustCode C(4) ADD COLUMN cCustId C(3)

  * Assign values to customer code that will sequentially append to the 
  * records already in the customer table
    UPDATE (lcTempFile) SET cCustCode = PADL(lnMax+RECNO(), 4, '0')
    lcTempStem = JUSTSTEM(lcTempFile)

  ENDIF

* Merge data into permanent file and then close file
  SELECT OrigFile
  APPEND FROM (lcTempFile) FOR !DELETED()
  USE

* Delete temporary data file
  IF USED(lcTempStem)
    USE IN (lcTempStem)
  ENDIF
  ERASE (lcTempStem)

RETURN


**************************************************
PROCEDURE CheckRunStatus
*-------------------------------------------------------
* Procedure....:  CheckRunStatus
* Written by...:  Michael P. Antonovich, Friday, April 28, 1998
*
* Abstract.....:  Adds a new record to the RunStatus table at the 
*                 start of each process
*
* Returns......:  .T. if task is not complete
*                 .F. if task has been completed and should NOT be rerun
*
* Parameters...:  cStreamId - Template Id to record
*                 cProcessId  - Process Id to record
*
*-------------------------------------------------------
LPARAMETERS lcStreamId, lcProcessId, lnCtr
LOCAL lnSelect, lcJobId, llSuccess, llBadRun, lnAns

* Store current work area and open the table RUNSTATUS
  lnSelect = SELECT()
  = OPENFILE("RunStatus")

  IF lnCtr = 1
  * check if there are any records for this stream that were not complete and  
    LOCATE FOR cStreamId + cProcessId = ;
               lcStreamId + lcProcessId AND ;
               !lSuccess AND ;
               !lArchived
    BadRun = FOUND()
  ELSE
    BadRun = .F.
  ENDIF

  IF BadRun
    lcJobId = RunStatus.cJobId
    oApp.ErrStr = "There was an error the last time this process was run.  " + ;
                  CHR(13) + CHR(10) + ;
                  "Do you want to archive the records for job " + ;
                  ALLTRIM(lcJobId) + "?" + CHR(13) + CHR(10) + ;
                  "Before running this stream again."
    IF oApp.Silent
    * If in silent mode automatically update the archival flag
*      WAIT WINDOW oApp.ErrStr TIMEOUT 60
      lnAns = IDYES
    ELSE
    * If in manual mode, prompt user whether the archival flag should be set
    * (in other words, has the error been corrected)
      lnAns = MESSAGEBOX(oApp.ErrStr, MB_ICONQUESTION+MB_YESNO, "Process Error")
    ENDIF

  * If in silent mode or the user answers yes, archive all incomplete records.
    IF lnAns = IDYES
      UPDATE RUNSTATUS ;
         SET lArchived = .T. ;
       WHERE cStreamId + cProcessId = ;
             lcStreamId + lcProcessId AND ;
             !lSuccess AND
             !lArchived
      llSuccess = .T.
    ELSE
      llSuccess = .F.
    ENDIF
    
  ELSE

  * Get next available job id and store in the application object
    IF lnCtr = 1
      SELECT MAX(cJobId) AS LastId FROM AUTOMATION!RunStatus INTO CURSOR NewJob
      oApp.AppJobId = PADL(INT(VAL(NewJob.LastId))+1, LEN(RunStatus.cJobId), "0")
      USE IN NewJob
    ENDIF

  * Insert first record for this template run.    
    INSERT INTO AUTOMATION!RunStatus (cJobId, cStreamId, cUserId, cProcessId, dStart) ;
           VALUES (oApp.AppJobId, lcStreamId, MyQueue.cUserId, lcProcessId, DATETIME())

    llSuccess = .T.
  ENDIF  

* Return to default work area
  SELECT (lnSelect)
RETURN llSuccess
  

*******************************************************
PROCEDURE SetRunStatus
*-------------------------------------------------------
* Procedure....:  SetRunStatus
* Written by...:  Michael P. Antonovich, April 25, 1998
*
* Abstract.....:  Updates an existing record in the RunStatus table for each completed process
*
* Returns......:  None
*
* Parameters...:  cStreamId - Template Id to record
*                 cProcessId  - Process Id to record
*
*-------------------------------------------------------
LPARAMETERS lcStreamId, lcProcessId

  UPDATE AUTOMATION!RunStatus ;
     SET dEnd     = DATETIME(), ;
         lSuccess = .T. ;
    WHERE cJobId      = oApp.AppJobId AND ;
          cStreamId   = lcStreamId AND ;
          cUserId     = MyQueue.cUserId AND ;
          cProcessId  = lcProcessId
RETURN  


*******************************************************
PROCEDURE RunQueue
*-------------------------------------------------------
* Procedure....:  RunQueue
* Written by...:  Michael P. Antonovich, April 28, 1998
*
* Abstract.....:  Reads RUNQUEUE to determine if there are other jobs to be run
*
* Returns......:  None
*
* Parameters...:  None
*
*-------------------------------------------------------
* Check if there are any jobs waiting in the queue
  lcCurArea = SELECT()
  SELECT * ;
    FROM RunQueue ;
   WHERE EMPTY(dRun) AND !lInProcess ;
   ORDER BY dScheduled ;
    INTO CURSOR MYQUEUE

  IF _TALLY > 0
  * Scan through all the waiting jobs
    SCAN
    * Mark the job as in process and call the driver program with the stream id
      lcQueueId  = MYQUEUE.cQueueId
      lcStreamId = MYQUEUE.cStreamId
      UPDATE RunQueue ;
         SET lInProcess = .T. ;
       WHERE cQueueId = lcQueueId
      DO DRIVER WITH lcStreamId
      IF oApp.Success
        UPDATE RunQueue ;
           SET lInProcess = .F., ;
               dRun = DATETIME() ;
         WHERE cQueueId = lcQueueId
      ENDIF
    ENDSCAN
  ENDIF

* Return to the original work area
  = CLOSFILE("MYQUEUE")
  SELECT (lcCurArea)
RETURN